program CDChecker;

uses
  QForms, SysUtils,
  Main in 'Main.pas' {frmMain} {$IFDEF MSWINDOWS}, Windows{$ENDIF};

{$R *.res}

{$IFDEF MSWINDOWS}
var
  hPrevious    : THandle;
  aAtom        : TAtom;
  i            : integer;
  sParams      : string;



{----------------------------------------------------------------------------------
 Description    : a callback function that receives top-level window handles as a
                  result of a call to the EnumWindows
 Parameters     : see notes
 Error checking : ?
 Notes          : check "EnumWindows" and "EnumWindowsProc" WinAPI help topics
-----------------------------------------------------------------------------------}
function EnumWindowsFunc(Handle: HWND; Param: LParam): boolean; stdcall;

  {---------------------------------------}
  function IsAPrevInstanceRunning: boolean;
  var
    ClassName : array[0..30] of Char;
    iAppID    : integer;
  begin

    { get the class name for the top-level window specified by Handle }
    GetClassName(Handle, ClassName, 30);

    { get the APP_ID for the application where the top-level window specified by Handle belongs }
    iAppID := SendMessage(Handle, UM_APP_ID_CHECK, 0, 0);

    { if ClassName matches to the this application's Main Form class name...}
    { and iAppID matches to to the this application's APP_ID}
    Result :=  (StrIComp(ClassName, MAIN_FORM_CLASS) = 0) and (iAppID = APP_ID);
  end;
  {---------------------------------------}
  
begin
  { must be True to continue the enumeration }
  Result := not IsAPrevInstanceRunning;
  if not Result then hPrevious := Handle;
end;

{----------------------------------------------------------------------------------
 Description    : restores the previous instance and passes the params to it
 Parameters     : none
 Error checking : YES
 Notes          : check "Atoms" WinAPI help topic
-----------------------------------------------------------------------------------}
procedure PassParamsToPreviousInstance;
begin
  PostMessage(hPrevious, UM_RESTORE_APPLICATION, 0, 0);
  if (ParamCount > 0) then
  begin
    aAtom := GlobalAddAtom(PChar(sParams));
    if aAtom = 0
    then RaiseLastWin32Error
    else
      begin
        SendMessage(hPrevious, UM_PREVIOUS_INST_PARAMS, aAtom, 0);
        GlobalDeleteAtom(aAtom);
      end;
  end;
end;

{----------------------------------------------------------------------------------
 Description    : program's main body
 Parameters     :
 Error checking :
 Notes          :
-----------------------------------------------------------------------------------}
begin
  hPrevious := 0;
  aAtom     := 0;
  EnumWindows(@EnumWindowsFunc, 0);

  sParams := '';

  { this is here because some long file names contain spaces }
  if ParamCount > 0 then
    for i := 1 to ParamCount do sParams := sParams + ' ' + ParamStr(i);


  sParams := Trim(sParams);


  { if a previous instance is already running then pass the command line
    parameters to it and exit... }
  if hPrevious <> 0
  then
    begin
      PassParamsToPreviousInstance;
      Exit;
    end
  else
    { ...else run the app normaly and handle the params }
    {$ENDIF}
    begin
      Application.Initialize;
      Application.Title := 'CD Checker';
      Application.ShowMainForm := False;
      Application.CreateForm(TfrmMain, frmMain);
      {$IFDEF MSWINDOWS}if ParamCount > 0 then frmMain.HandleParams(sParams);{$ENDIF}
      Application.Run;
    {$IFDEF MSWINDOWS}end;{$ENDIF}


end.

