

{

CD Checker
by Antti Kirjavainen

You can do what you want with this code.
Code is 100 % cross-platform compatible and
compiles on both, Delphi and Kylix.

}


unit Main;

interface

uses
  SysUtils, Variants, Classes, QGraphics,
  QDialogs, INIFiles, QExtCtrls, QMenus, QTypes, QStdCtrls, QControls
  {$IFDEF MSWINDOWS}, Windows, ShellAPI, FileCtrl, Registry, MMSystem, Messages, Forms{$ENDIF}
  {$IFDEF LINUX}, Libc, QForms{$ENDIF};

const
  ENTER = Chr(13) + Chr(10);
  {$IFDEF MSWINDOWS}
  WM_ICONTRAY = WM_USER + 1;

 MAIN_FORM_CLASS = 'TfrmMain';

  { UM stands for User Message }
  UM_APP_ID_CHECK                = WM_APP + 101;
  UM_RESTORE_APPLICATION         = WM_APP + 102;
  UM_PREVIOUS_INST_PARAMS        = WM_APP + 103;

  { a user defined unique app id }
  APP_ID                         = 25038308;
  {$ENDIF}

type
  TfrmMain = class(TForm)
    mnuMain: TMainMenu;
    mnuFile: TMenuItem;
    mnuCheck: TMenuItem;
    N1: TMenuItem;
    mnuExit: TMenuItem;
    mnuHelp: TMenuItem;
    mnuAbout: TMenuItem;
    grpSettings: TGroupBox;
    chkEject: TCheckBox;
    chkStart: TCheckBox;
    mnuPopUp: TPopupMenu;
    mnuCheck2: TMenuItem;
    N2: TMenuItem;
    mnuHomepage2: TMenuItem;
    mnuEmail2: TMenuItem;
    mnuAbout2: TMenuItem;
    N3: TMenuItem;
    mnuExit2: TMenuItem;
    N4: TMenuItem;
    mnuHomepage: TMenuItem;
    mnuEmail: TMenuItem;
    tmrUpdateTray: TTimer;
    txtDrive: TEdit;
  procedure AppException(Sender: TObject; E: Exception);
    procedure FormCreate(Sender: TObject);
    procedure mnuExitClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure chkEjectClick(Sender: TObject);
    procedure chkStartClick(Sender: TObject);
    procedure mnuCheckClick(Sender: TObject);
    procedure mnuAboutClick(Sender: TObject);
    procedure AppMinimize(Sender: TObject);
    procedure mnuHomepageClick(Sender: TObject);
    procedure mnuEmailClick(Sender: TObject);
    procedure tmrUpdateTrayTimer(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure txtDriveChange(Sender: TObject);
    procedure txtDriveKeyPress(Sender: TObject; var Key: Char);

  private
    { Private declarations }
  {$IFDEF MSWINDOWS}
  a:NOTIFYICONDATA;
    procedure UMAppIDCheck(var Message : TMessage); message UM_APP_ID_CHECK;
    procedure UMRestoreApplication(var Message : TMessage); message UM_RESTORE_APPLICATION;
    procedure UMPreviousInstParams(var Message : TMessage); message UM_PREVIOUS_INST_PARAMS;
  {$ENDIF}
  public
    { Public declarations }
    {$IFDEF MSWINDOWS}
    procedure HandleParams(S : string);
    procedure TrayMessage(var Msg: TMessage); message WM_ICONTRAY;
    {$ENDIF}
 end;

var
  frmMain: TfrmMain;
  INIFileName : String;
  {$IFDEF LINUX}
        sHomeFolder: String;
        sAutoStartLink: String;
  {$ENDIF}
  {$IFDEF MSWINDOWS}FWindowHandle: HWND;{$ENDIF}

  function ReadFromINI(const Section : String; const Key : String; const Default : String) :String; overload;
  function ReadFromINI(const Section : String; const Key : String; const Default : LongInt) :LongInt; overload;
  function WriteToINI(const Section : String; const Key : String; const Value : String) :Boolean; overload;
  function WriteToINI(const Section : String; const Key : String; const Value : LongInt) :Boolean; overload;
  function BooleanToInt(const BoolOp : Boolean) :Integer;
  function IntToBoolean(const IntOp : Integer) :Boolean;
  procedure CheckCD;
  function OpenCD(Drive : String) : Boolean;
  function CloseCD(Drive : String) : Boolean;


implementation

{$R *.dfm}

procedure TfrmMain.AppException(Sender: TObject; E: Exception);
begin
//Nothing
end;

{$IFDEF MSWINDOWS}
function OpenCD(Drive : String) : Boolean;
var
        Res : MciError;
        OpenParm: TMCI_Open_Parms;
        Flags : DWord;
        S : String;
        DeviceID : Word;

begin

  Result := False;
  S := Drive + ':';
  Flags := mci_Open_Type or mci_Open_Element;

  with OpenParm do begin
    dwCallback := 0;
    lpstrDeviceType := 'CDAudio';
    lpstrElementName := PChar(S);
  end;

  Res := mciSendCommand(0, mci_Open, Flags, Longint(@OpenParm));
  IF (Res <> 0) then exit;
  DeviceID := OpenParm.wDeviceID;

  try
    Res:=mciSendCommand(DeviceID, MCI_SET, MCI_SET_DOOR_OPEN, 0);
    if (Res = 0) then exit;
    Result := True;
  finally
    mciSendCommand(DeviceID, mci_Close, Flags, Longint(@OpenParm));
  end;

end;

function CloseCD(Drive : String) : Boolean;
var
        Res : MciError;
        OpenParm: TMCI_Open_Parms;
        Flags : DWord;
        S : String;
        DeviceID : Word;

begin
  Result := False;
  S := Drive + ':';
  Flags := mci_Open_Type or mci_Open_Element;

  With OpenParm do begin
    dwCallback := 0;
    lpstrDeviceType := 'CDAudio';
    lpstrElementName := PChar(S);
  end;

  Res := mciSendCommand(0, mci_Open, Flags, Longint(@OpenParm));
  IF (Res <> 0) Then exit;
  DeviceID:=OpenParm.wDeviceID;

  try
    Res := mciSendCommand(DeviceID, MCI_SET, MCI_SET_DOOR_CLOSED, 0);
    IF (Res = 0) Then exit;
    Result := True;
  finally
    mciSendCommand(DeviceID, mci_Close, Flags, Longint(@OpenParm));
  end;

end;

{----------------------------------------------------------------------------------}
procedure TfrmMain.UMAppIDCheck(var Message: TMessage);
begin
  Message.Result := APP_ID;
end;
{----------------------------------------------------------------------------------}
procedure TfrmMain.UMRestoreApplication(var Message: TMessage);
begin
  if IsIconic(Application.Handle) then Application.Restore else Application.BringToFront;
  if not Self.Visible then Self.Show;
end;
{----------------------------------------------------------------------------------}
procedure TfrmMain.UMPreviousInstParams(var Message: TMessage);
var
 Len : integer;
 S   : string;
begin
  SetLength(S, MAX_PATH);
  Len := GlobalGetAtomName(Message.wParam, PChar(S), MAX_PATH);
  if Len = 0
  then RaiseLastWin32Error
  else
    begin
      SetLength(S, Len);
      HandleParams(S);
    end;
end;
{----------------------------------------------------------------------------------}
procedure TfrmMain.HandleParams(S: string);
begin
//
end;
{----------------------------------------------------------------------------------}

procedure TfrmMain.TrayMessage(var Msg: TMessage);
var
   curPos : TPoint;

begin
   case Msg.lParam of

      WM_RBUTTONUP:
         begin
         // popup menu
         GetCursorPos(curPos);
         mnuPopUp.Popup(curPos.x, curPos.y);
         end;

      WM_LBUTTONDBLCLK:
         begin
         Self.Show;
         Application.Restore;
         Application.BringToFront;
         end;

      {WM_LBUTTONDOWN:
         begin
         end;}
      end;
end;
{$ENDIF}

{$IFDEF LINUX}
function OpenCD(Drive: String) : Boolean;
var
        sCommand: String;

begin
sCommand := 'eject ' + Drive + '&';
if (Libc.system(PChar(sCommand)) = 0) then Result := True else Result := False; 
end;

function CloseCD(Drive: String) : Boolean;
var
        sCommand: String;

begin
sCommand := 'eject -t ' + Drive + '&';
if (Libc.system(PChar(sCommand)) = 0) then Result := True else Result := False; 
end;
{$ENDIF}

function ReadFromINI(const Section : String; const Key : String; const Default : String) :String;
var
ReadINI: TIniFile;

begin

try
        ReadINI := TIniFile.Create(INIFileName);
        ReadFromINI := ReadINI.ReadString(Section, Key, Default);
finally
        ReadINI.Free;
end;

end;

function ReadFromINI(const Section : String; const Key : String; const Default : LongInt) :LongInt;
var
ReadINI: TIniFile;

begin

try
        ReadINI := TIniFile.Create(INIFileName);
        ReadFromINI := ReadINI.ReadInteger(Section, Key, Default);
finally
        ReadINI.Free;
end;

end;


function WriteToINI(const Section : String; const Key : String; const Value : String) :Boolean;
var
WriteINI: TIniFile;

begin

try
        WriteINI := TIniFile.Create(INIFileName);
        WriteINI.WriteString(Section, Key, Value);
        WriteToINI := True;
finally
        WriteINI.Free;
end;

end;

function WriteToINI(const Section : String; const Key : String; const Value : LongInt) :Boolean;
var
WriteINI: TIniFile;

begin

try
        WriteINI := TIniFile.Create(INIFileName);
        WriteINI.WriteInteger(Section, Key, Value);
        WriteToINI := True;
finally
        WriteINI.Free;
end;

end;

function BooleanToInt(const BoolOp : Boolean) :Integer;
begin
if BoolOp then BooleanToInt := 1 else BooleanToInt := 0;
end;

function IntToBoolean(const IntOp : Integer) :Boolean;
begin
if (IntOp > 0) then IntToBoolean := True else IntToBoolean := False;
end;



procedure CheckCD;
var
        Drv: TStringList;
        d: Integer;

        {$IFDEF LINUX}
        sCommand: String;
        iStatus: Integer;
        {$ENDIF}

begin

// Check every drive...

try
                                Drv := TStringList.Create;
                                Drv.Text := frmMain.txtDrive.Text;
                                Drv.Text := StringReplace(Drv.Text, ' ', '', [rfReplaceAll]);
                                Drv.Text := StringReplace(Drv.Text, ':', '', [rfReplaceAll]);
                                Drv.Text := StringReplace(Drv.Text, ';', ENTER, [rfReplaceAll]);
                                Drv.Text := StringReplace(Drv.Text, ',', ENTER, [rfReplaceAll]);


                                for d := 0 to Drv.Count - 1 do begin
                                        {$IFDEF LINUX}
                                        sCommand := 'umount ' + Drv.Strings[d];
                                        Libc.system(PChar(sCommand));

                                        sCommand := 'mount ' + Drv.Strings[d];
                                        iStatus := Libc.system(PChar(sCommand));
                                        if (iStatus = 0) then begin
                                        {$ENDIF}
                                        {$IFDEF MSWINDOWS}if DirectoryExists(Drv.Strings[d] + ':\') then begin{$ENDIF}
                                                                                        if (frmMain.chkEject.State = cbChecked) then OpenCD(Drv.Strings[d]);
                                                                                        MessageDlg('There is a disc in the drive ' + Drv.Strings[d] + ':' + ENTER + ENTER + 'Press OK to continue.', mtInformation, [mbOK], 0);
                                                                                        if (frmMain.chkEject.State = cbChecked) then CloseCD(Drv.Strings[d]);
                                                                                        end;
                                end;

finally
                                Drv.Free;
end;

end;

procedure TfrmMain.FormCreate(Sender: TObject);
{$IFDEF MSWINDOWS}
var
        reg : TRegIniFile;
{$ENDIF}

begin
Application.OnException := AppException;
Application.OnMinimize := AppMinimize;

{$IFDEF MSWINDOWS}INIFileName := ExtractFilePath(Application.EXEName) + 'CDCheck.ini';{$ENDIF}
{$IFDEF LINUX}
sHomeFolder := IncludeTrailingBackSlash(getenv('HOME'));
sAutoStartLink := sHomeFolder + '.kde/Autostart/CDCheck';
INIFileName := sHomeFolder + '.CDCheck.ini';
{$ENDIF}

if not FileExists(INIFileName) then begin
                                    Self.Show();
                                    Application.BringToFront();
                                    WriteToINI('Options', 'Settings', 1);
                                    {$IFDEF MSWINDOWS}MessageDlg('Seems that this is the first time you start CD Checker!' + ENTER + 'You can set the settings now and then minimize the program.' + ENTER + ENTER + 'You will find the icon from the system tray, near the clock.' + ENTER + 'To see this window again, just double-click the icon.', mtInformation, [mbOK], 0);{$ENDIF}
                                    {$IFDEF LINUX}MessageDlg('Seems that this is the first time you start CD Checker!' + ENTER + 'You can set the settings now and then hide the program by minimizing.' + ENTER + ENTER + 'To get back to this setting window, start the program with command-line switch "-s".', mtInformation, [mbOK], 0);{$ENDIF}
                                    end;

if (Pos('-s', LowerCase(ParamStr(1))) > 0) then Self.Show();

{$IFDEF LINUX}
txtDrive.Text := ReadFromINI('Settings', 'Drive', '/mnt/cdrom');
chkStart.Caption := '&Start with KDE';
if (FileExists(sAutoStartLink)) then chkStart.Checked := True else chkStart.Checked := False;
{$ENDIF}


if (ReadFromINI('Settings', 'Eject', 1) = 1) then chkEject.State := cbChecked else chkEject.State := cbUnchecked;

{$IFDEF MSWINDOWS}
tmrUpdateTray.Enabled := True;
txtDrive.Text := ReadFromINI('Settings', 'Drive', 'E, F');
chkStart.Caption := '&Start with Windows';

reg := TRegIniFile.Create('');
reg.RootKey := HKEY_CURRENT_USER;
if (reg.ReadString('Software\Microsoft\Windows\CurrentVersion\Run', 'CDCheck', '')) = Application.ExeName then chkStart.State := cbChecked;
reg.Free;

a.cbSize := sizeof(a);
a.Wnd := Handle;
a.uID := 0;
StrPCopy(a.szTip, txtDrive.Text + ' - ' + 'CD Checker');
a.uCallbackMessage := WM_USER + 1;
a.hIcon := Application.Icon.Handle;
a.uFlags :=  NIF_ICON + NIF_MESSAGE + NIF_TIP;
Shell_NotifyIcon(NIM_ADD, @a);
{$ENDIF}


end;

procedure TfrmMain.mnuExitClick(Sender: TObject);
begin
try
        {$IFDEF MSWINDOWS}Shell_NotifyIcon(NIM_DELETE,@a);{$ENDIF}
        Application.Terminate;
except
end;
end;

procedure TfrmMain.FormResize(Sender: TObject);
begin
if (Self.WindowState = wsMinimized) then Self.Visible := False;
end;





procedure TfrmMain.chkEjectClick(Sender: TObject);
begin
if chkEject.State = cbChecked then WriteToIni('Settings', 'Eject', 1) else WriteToIni('Settings', 'Eject', 0);;
end;

procedure TfrmMain.chkStartClick(Sender: TObject);
{$IFDEF LINUX}
var
        sLink: TStringList;

begin
if (chkStart.State = cbChecked) then begin
                                     try
                                                sLink := TStringList.Create();
                                                sLink.Text := '[Desktop Entry]' + ENTER + 'Exec=' + Application.ExeName + ' -ns' + ENTER + 'Name=CDChecker' + ENTER + 'Type=Application';
                                                sLink.SaveToFile(sAutoStartLink); 
                                                sLink.Free();
                                     except
                                                MessageDlg('Could not write the shortcut ' + sAutoStartLink + '!', mtError, [mbOK], 0);
                                                chkStart.Enabled := False;
                                                sLink.Free();
                                     end;
                                     end
                                     else begin
                                     DeleteFile(sAutoStartLink);
                                     end;

{$ENDIF}
{$IFDEF MSWINDOWS}
var
        reg : TRegIniFile;

begin

if (chkStart.State = cbChecked) then begin
                                reg := TRegIniFile.Create('');
                                reg.RootKey := HKEY_CURRENT_USER;
                                reg.WriteString('Software\Microsoft\Windows\CurrentVersion\Run', 'CDCheck', Application.ExeName);
                                reg.Free;
                                end
                                else begin
                                reg := TRegIniFile.Create('');
                                reg.RootKey := HKEY_CURRENT_USER;
                                reg.DeleteKey('Software\Microsoft\Windows\CurrentVersion\Run', 'CDCheck');
                                reg.Free;
                                end;

{$ENDIF}
end;

procedure TfrmMain.mnuCheckClick(Sender: TObject);
begin
CheckCD();
end;

procedure TfrmMain.AppMinimize(Sender: TObject);
begin
Self.Hide();
end;

procedure TfrmMain.mnuAboutClick(Sender: TObject);
begin
MessageDlg('CD Checker v3.00' + ENTER + 'by Antti Kirjavainen' + ENTER + ENTER + 'Freeware.' + ENTER, mtInformation, [mbOK], 0);
end;

procedure TfrmMain.mnuHomepageClick(Sender: TObject);
begin
{$IFDEF MSWINDOWS}ShellExecute(Handle, 'open', 'http://www.htk.fi/public/akirjavainen', '', '', SW_SHOWDEFAULT);{$ENDIF}
{$IFDEF LINUX}Libc.system('konqueror http://www.htk.fi/public/akirjavainen');{$ENDIF}
end;

procedure TfrmMain.mnuEmailClick(Sender: TObject);
begin
{$IFDEF MSWINDOWS}ShellExecute(Handle, 'open', 'mailto:antti.kirjavainen@mail.htk.fi', '', '', SW_SHOWDEFAULT);{$ENDIF}
{$IFDEF LINUX}Libc.system('konqueror mailto:antti.kirjavainen@mail.htk.fi');{$ENDIF}
end;

procedure TfrmMain.tmrUpdateTrayTimer(Sender: TObject);
// Explorer crashes quite often, so make sure the icon stays in the tray (refresh):
begin
{$IFDEF MSWINDOWS}Shell_NotifyIcon(NIM_ADD, @a);{$ENDIF}
end;

procedure TfrmMain.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
try
        CanClose := False;
        {$IFDEF MSWINDOWS}Shell_NotifyIcon(NIM_DELETE,@a);{$ENDIF}
        CheckCD();
except
end;

CanClose := True;

end;

procedure TfrmMain.txtDriveChange(Sender: TObject);
begin
{$IFDEF MSWINDOWS}
txtDrive.Text := UpperCase(txtDrive.Text);
StrPCopy(a.szTip, txtDrive.Text + ' - ' + 'CD Checker');
Shell_NotifyIcon(NIM_MODIFY,@a);
{$ENDIF}

WriteToINI('Settings', 'Drive', txtDrive.Text);
end;

procedure TfrmMain.txtDriveKeyPress(Sender: TObject; var Key: Char);
begin
if (Key = 'C') or (Key = 'A') or (Key = 'B') or (Ord(Key) = 13) or (Ord(Key) = 10) then begin
                Key := Char(0);
                exit;
                end;
end;

end.
